//
// Jap to Eng Dictionary Registration Program /*  */
//
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "macro.h"

#define RecNb   32
#define WdNbPg  280
#define WdDisp  30
#define Size    256

FILE *jed;
extern FILE *dbg;
char *jDicfile = "JEDIC.J2E";

char jer[Size];
extern char aj[][YakugoMax][JWordLen], az[][YakugoMax][ZokuLen];
extern char caj[][JWordLen], caz[][ZokuLen], wd[];
extern char jwd[], wdw[], wdx[], wdh[], zkc[];
extern char Dbg;
extern int  cnb, ct;
int  wnb;
char godan_mize[][3] = {"","","","","","","","",""};
char godan_masu[][3] = {"","","","","","","","","",""}; //
char godan_kate[][3] = {"","","","","","","","","","","",""}; // 䥂ł
char godan_syuu[][3] = {"","","","","","","","",""};
char godan_miz2[][3] = {"","","","","","","","",""};
char ippan_2chr[][3] = {"","","","","","",""};
// ㉺iꊲꎚ̏ꍇ
char ichi_gokan[][3] = {"","","","","","","","","","",
                        "","","",
                        "","","o","Q","o"}; // 18

char kago[][11] = {"Ȃ","܂","","悤","","","Ȃ","","","","܂","Ȃ","","܂","ςȂ",""}; // 16
char kag3[][11] = {"Ȃ","܂","","悤","","","Ȃ","","","","܂","Ȃ","","܂","ςȂ",""}; // 16
char sahe[][11] = {"Ȃ","܂","","","","Ȃ","",
		   "","","","܂","Ȃ","܂","ςȂ","","₷","","","","o","n",  "Ȃ","炵","","","A","","悤","ɂ","Â炢"}; // 30
char sazu[][11] = {"Ȃ","","","","","Ȃ","","","Ȃ","",
                   "","悤"}; // 12
char kag2[][5] = {"","","",""};
char sah2[][5] = {"","","",""};
char saz2[][5] = {"","","",""}; // 
char sahz[][5] = {"|","p","|","|"};

// :make邹悹 Fꂽ
char go_a[][11] = {"Ȃ","","","","","Ȃ","","˂",""}; // 9
//  ȂȂȂ
char go_i[][11] = {"܂","","","","܂",
                   "","܂","ςȂ","","₷",
                   "","ɂ","˂","","",
                   "","͂","","","","","","o","","","A"}; // 26 ܂
char go_e[][11] = {"","","Ȃ","Ȃ","","","܂","Ă","B","v"}; // 10  Kp
char gz_e[][ 5] = {"\0","|","|","|","|","|","|","|","|","|"}; // 10 i͓Kp
char go_o[][11] = {""}; // 1

char kako[][5] = {"","","","","",""}; // 6
char site[][5] = {"","","","","",""};
char genz[][3] = {"","","","","",""};

char ke_i[][11] = {"낤","","",""}; // 4
char keDa[][11] = {"낤","ł","ɂ","ɂ","ɂ",""}; // 6

char *genSkip8 = "ɂ,Ƃ,オ,낦,悤ɂ,ߖ,܂,炩,΂,̂߂,||,낤,C^r,Ƃ͂,g,ڂW,߂,F,ɂĂ,vƂ";
char *genSkip6 = "ύX,,b,,M,],΂ߏ,ʂ,Ȃ,܂,ɂ,,߂,lC,,₦,v`,v,,Ȃ,ɂ͂,E,₢,mb,[,,S,c_,肵,F肵,ᔽ,,Ƃ,y,O,\\,S,A,,A";

// 
//  ł
// o^ ł ł 
// o^ ł
char desu[][18] = {  };
char    bungo[14][18] = {"ł","ł","",
	"Ȃ","͂Ȃ","Ȃ","͂Ȃ",
	"͖","Ȃ","",
	"","","낤","ɂȂ"},
 	kougo[14][20] = {"ł","ł","ł","͂܂ł",
	"͂܂","܂","͂܂ł","͗L܂ł",
	"܂ł","L܂ł","܂","܂",
	"ł傤","ɂ܂ł"};

symbck()
{
    char mj;
    ct = 0;
    if(strcmp(jwd, "B") == 0){
      strcpy(caj[0], "0.");    strcpy(caz[0], ".");    ct = 1;    return;
    }
    if(strcmp(jwd, "") == 0){
      strcpy(caj[0], "0.");    strcpy(caz[0], ".");    ct = 1;    return;
    }
    if(strcmp(wdw, "B") == 0){  strcpy(jwd, wdw);
      strcpy(caj[0], "??");    strcpy(caz[0], "?");    ct = 1;    return;
    }
    if(strcmp(wdw, "H") == 0){  strcpy(jwd, wdw);
      strcpy(caj[0], "??");    strcpy(caz[0], "?");    ct = 1;    return;
    }
    if(strcmp(wdw, "") == 0){  strcpy(jwd, wdw);
      strcpy(caj[0], "??");    strcpy(caz[0], "?");    ct = 1;    return;
    }
    //    if(strcmp(wdw, "́B") == 0){  strcpy(jwd, wdw);
    //      strcpy(caj[0], "??");    strcpy(caz[0], "?");    ct = 1;    return;
    //    }
    if(strcmp(wdw, "ˁH") == 0){  strcpy(jwd, wdw);
      strcpy(caj[0], "0?");    strcpy(caz[0], "?|t");    ct = 1;    return;
    }
    if(strcmp(jwd, "H") == 0){
      strcpy(caj[0], "??");    strcpy(caz[0], "?");    ct = 1;    return;
    }
    if(strcmp(jwd, "I") == 0){
      strcpy(caj[0], "!!");    strcpy(caz[0], "!");    ct = 1;    return;
    }
    if(strcmp(jwd, "A") == 0){
      strcpy(caj[0], "0,");    strcpy(caz[0], ",");    ct = 1;    return;
    }
    if(strcmp(jwd, "E") == 0){
      strcpy(caj[0], "0-");    strcpy(caz[0], "");    ct = 1;    return;
    }
    if(strcmp(jwd, "q") == 0){
      strcpy(caj[0], "0<");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "r") == 0){
      strcpy(caj[0], "0>");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "u") == 0){
      strcpy(caj[0], "0\'");    strcpy(caz[0], "|u");    ct = 1;    return;
    }
    if(strcmp(jwd, "v") == 0){
      strcpy(caj[0], "0\'");    strcpy(caz[0], "|v");    ct = 1;    return;
    }
    if(strcmp(jwd, "g") == 0){
      strcpy(caj[0], "0\"");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "h") == 0){
      strcpy(caj[0], "0\"");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "w") == 0){
      strcpy(caj[0], "0\"");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "x") == 0){
      strcpy(caj[0], "0\"");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "i") == 0 || strcmp(jwd, "(") == 0){
      strcpy(caj[0], "0(");    strcpy(caz[0], "(");    ct = 1;    return;
    }
    if(strcmp(jwd, "j") == 0 || strcmp(jwd, ")") == 0){
      strcpy(caj[0], "0)");    strcpy(caz[0], ")");    ct = 1;    return;
    }
    if(strcmp(jwd, "@") == 0 || strcmp(jwd, " ") == 0){
      strcpy(caj[0], "0 ");    strcpy(caz[0], ",");    ct = 1;    return;
    }
    if(jwd[0] == '%'){
      strcpy(caj[0], jwd);    strcpy(caz[0], jwd);    ct = 1;    return;
    }
    mj = jwd[0];
    if(mj == '#' || mj == '*'){
      caj[0][0] = mj;
      caj[0][1] = '\0';
      strcpy(caz[0], jwd);
      ct = 1;
      return;
    }
    if(strcmp(jwd, "[") == 0){
      strcpy(caj[0], "0[");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "]") == 0){
      strcpy(caj[0], "0]");    strcpy(caz[0], "|");    ct = 1;    return;
    }
    if(strcmp(jwd, "F") == 0){
      strcpy(caj[0], "0:");    strcpy(caz[0], "");    ct = 1;    return;
    }
    if(isdigit(*jwd)){
      alnum();    strcpy(caz[0], "n|");   ct = 1;   return;
    }
    if(isalnum(*jwd)){
      alnum();    strcpy(caz[0], "|");   ct = 1;   return;
    }
//    if(strcmp(jwd, "O") >= 0 && strcmp(jwd, "X") <= 0){
//      digit();    strcpy(caz[0], "n|");   ct = 1;   return;
//    }
}

numbck(){
    ct = 0; 
    if(strcmp(jwd, "O") >= 0 && strcmp(jwd, "X") <= 0){
      digit();      strcpy(caz[0], "n|");      ct = 1;   return;
    }
}

digit(){
    char hankaku[17], buf[3];
    int  i;
    for(i = 2; ; i=i+2){
      strncpy(buf, wdw+i, 2);   buf[2] = '\0';
      if(strcmp(buf, "O") >= 0 && strcmp(buf, "X") <= 0){
        strcat(jwd, buf);  
      }
      else break;
    }
    nzentohan(jwd, hankaku);
    strcpy(caj[0], "1");
    strcat(caj[0], hankaku);
}

alnum(){
    char buf[3];
    int  i;
    for(i = 1; ; i++){
      strncpy(buf, wdw+i, 1);   buf[1] = '\0';
      if(isalnum(buf[0]) || ispunct(buf[0])) strcat(jwd, buf);  
      else break;
    }
    strcpy(caj[0], "1");
    strcat(caj[0], jwd);
}

preEdit()
{
   if(kahenPred() == 1) return;
   if(sahenPred() == 1) return;
 if(Dbg == 'Y') fprintf(dbg, "--- preEdit --- desumasu \n");
   if(desumasu_ck() == 1) return;
}
preConj()
{


    //  if(Dbg == 'Y') fprintf(dbg, "--- preConj --- desumasu \n");
    //    if(desumasu_ck() == 1) return;

}

int postConj(jpw, hinsi)
     char *jpw, *hinsi;
{
  if(kahenKatsu(jpw) == 1) return 1;
  if(sahenKatsu(jpw) == 1) return 1;
  if(keiyoKatsu(jpw, hinsi) == 1) return 1;

  if(ichidanKatsu(jpw) == 1) return 1;
  if(kamidanKatsu(jpw) == 1) return 1;
  if(simodanKatsu(jpw) == 1) return 1;

  if(godanKatsu(jpw) == 1) return 1;
  if(pastKatsu(jpw) == 1) return 1;
  return 0;
}

int kahenPred()
{
  int i, yn = 0;
    if(strncmp(wdw, "", 2) == 0
    || strncmp(wdw, "", 2) == 0 || strncmp(wdw, "", 2) == 0 ){
        for(i = 0; i < 16; i++){ // kag3
          if(strncmp(wdw, kag3[i], strlen(kag3[i])) == 0){
    	      ptreplace(wdw, 2, "");
	      strcpy(jwd, "");
 if(Dbg == 'Y') fprintf(dbg, "--- kahenPred kag3 --- wdw = %s \n", wdw);
	      yn = 1;
	      break;
	  }
	}
	//    ͎ɓo^
    }
    return(yn);
}

int kahenKatsu(jpw)
char *jpw;
{
    int ln, i, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 4) return(yn);
    if(strcmp(jpw, "") == 0 || strcmp(jpw, "") == 0){
        for(i = 0; i < 16; i++){ // kago
          if(strncmp(wdw, kago[i], strlen(kago[i])) == 0){
	      ptreplace(wdw, 2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- kahenKatsu kago --- jpw = %s wdw = %s \n", jpw, wdw);
	      yn = 1;
	      break;
	  }
	}
        for(i = 0; i < 4; i++){
          if(strncmp(wdw, kag2[i], 4) == 0){
	      ptreplace(wdw, 4, "");
	      strcat(zkc, sahz[i]);  yn = 1;
	      break;
	  }
	}
    }
    return(yn);
}

int sahenPred()
{
  int i, yn = 0;
    if(strncmp(wdw, "", 2) == 0 || strncmp(wdw, "", 2) == 0
    || strncmp(wdw, "", 2) == 0 || strncmp(wdw, "", 2) == 0
    ){
        for(i = 0; i < 30; i++){ // sahe
          if(strncmp(wdw, sahe[i], strlen(sahe[i])) == 0){
    	      ptreplace(wdw, 2, "");
	      strcpy(jwd, "");
 if(Dbg == 'Y') fprintf(dbg, "--- sahenPred sahe --- wdw = %s \n", wdw);
	      yn = 1;
	      break;
	  }
	}
	//     ͎ɓo^
    }
    return(yn);
}

int sahenKatsu(jpw)
char *jpw;
{
    int ln, i, yn;
    char cmp[40];
    yn = 0;
    ln = strlen(jpw);
    if(ln < 4) return(yn);

    if(strcmp(jpw+ln-4, "") == 0){
      strncpy(cmp, jpw, ln-4);  cmp[ln-4] = '\0';
      if(strncmp(wdw, cmp, ln-4) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- sahenKatsu --- jpw = %s cmp = %s \n", jpw, cmp);
        for(i = 0; i < 30; i++){ // sahe
          if(strncmp(wdw+ln-4, sahe[i], strlen(sahe[i])) == 0){
	      ptreplace(wdw+ln-4, 2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- sahenKatsu sahe --- jpw = %s wdw = %s \n", jpw, wdw);
	      yn = 1;
	      break;
	  }
	}
        if(yn == 1 && strncmp(wdw+ln, "A", 2) == 0){ // 
   	    strcat(zkc, "|");
	    return(yn);
        }
        for(i = 0; i < 4; i++){
          if(strncmp(wdw+ln-4, sah2[i], 4) == 0){
	      ptreplace(wdw+ln-4, 4, "");
	      strcat(zkc, sahz[i]);  yn = 1;
	      break;
	  }
	}
      }
    } else
    if(strcmp(jpw+ln-4, "") == 0){ // _
      strncpy(cmp, jpw, ln-4);  cmp[ln-4] = '\0';
      if(strncmp(wdw, cmp, ln-4) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- sahenKatsu --- jpw = %s cmp = %s \n", jpw, cmp);
        for(i = 0; i < 12; i++){ // sazu
          if(strncmp(wdw+ln-4, sazu[i], strlen(sazu[i])) == 0){
	      ptreplace(wdw+ln-4, 2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- sahenKatsu sazu --- jpw = %s wdw = %s \n", jpw, wdw);
	      yn = 1;
	      break;
	  }
	}
        for(i = 0; i < 4; i++){
          if(strncmp(wdw+ln-4, saz2[i], 4) == 0){
	      ptreplace(wdw+ln-4, 4, "");
	      strcat(zkc, sahz[i]);  yn = 1;
	      break;
	  }
	}
      }
    }
    return(yn);
}

int ichidanKatsu(jpw)
char *jpw;
{
    int ln, i, j, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 4) return(yn);
    //    if(strncmp(wdw, "", 2) == 0) return(yn);
    if(strcmp(jpw+ln-2, "") == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
	for(j = 0; j < 18; j++){ // ichi_gokan
          if(ln == strlen(ichi_gokan[j])+2
          && strncmp(gokan, ichi_gokan[j], strlen(ichi_gokan[j])) == 0){
 if(Dbg == 'Y') fprintf(dbg, "--- ichidanKatsu --- jpw = %s gokan = %s ichi_gokan[j] = %s \n", jpw, gokan, ichi_gokan[j]);
            yn = ichiSearch("ichidan", jpw, ln);

            break;
	  }
	}
      }
    }
    return(yn);
}

int kamidanKatsu(jpw)
char *jpw;
{
    int ln, i, j, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 6) return(yn);
    if(strcmp(jpw+ln-2, "") == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
	for(j = 0; j < 10; j++){ // _masu
          if(strncmp(wdw+ln-4, godan_masu[j], 2) == 0){
 if(Dbg == 'Y') fprintf(dbg, "--- kamidanKatsu --- jpw = %s gokan = %s \n", jpw, gokan);
            yn = ichiSearch("kamidan", jpw, ln);
	    break;
	  }
	}
      }
    }
    return(yn);
}

int simodanKatsu(jpw)
char *jpw;
{
    int ln, i, j, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 6) return(yn);
    if(strncmp(wdw, "˂", 6) == 0) return(yn);
    if(strcmp(jpw+ln-2, "") == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
	for(j = 0; j < 12; j++){ // _kate
          if(strncmp(wdw+ln-4, godan_kate[j], 2) == 0){
 if(Dbg == 'Y') fprintf(dbg, "--- simodanKatsu --- jpw = %s gokan = %s \n", jpw, gokan);
            yn = ichiSearch("simodan", jpw, ln);

	    break;
	  }
	}
      }
    }
    return(yn);
}

int ichiSearch(mode, jpw, ln)
char *mode, *jpw;
int ln;
{
  int i, yn = 0;
            for(i = 0; i < 9; i++){ // go_a
              if(strncmp(wdw+ln-2, go_a[i], strlen(go_a[i])) == 0){
	        ptinsert(wdw+ln-2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- %sKatsu go_a --- jpw = %s wdw = %s \n", mode, jpw, wdw);
	        yn = 1;
	        return(yn);
	      }
	    }
            for(i = 0; i < 26; i++){ // go_i
              if(strncmp(wdw+ln-2, go_i[i], strlen(go_i[i])) == 0){
	        ptinsert(wdw+ln-2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- %sKatsu go_i --- jpw = %s wdw = %s \n", mode, jpw, wdw);
                if(strcmp(go_i[i], "A") == 0)    strcat(zkc, "|");
	        yn = 1;
		return(yn);
	      }
	    }
            for(i = 0; i < 10; i++){ // go_e
              if(strncmp(wdw+ln-2, go_e[i], strlen(go_e[i])) == 0){
	        ptinsert(wdw+ln-2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- %sKatsu go_e --- jpw = %s wdw = %s \n", mode, jpw, wdw);
	        yn = 1;
		return(yn);
	      }
	    }

            for(i = 0; i < 1; i++){ // go_o
              if(strncmp(wdw+ln, go_o[i], strlen(go_o[i])) == 0){
	        ptinsert(wdw+ln-2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- %sKatsu go_o --- jpw = %s wdw = %s \n", mode, jpw, wdw);
	        yn = 1;
	        break;
	      }
	    }
            if(yn == 1 && strncmp(wdw+ln, "", 2) == 0){ // 
	      ptcut(wdw+ln, 2);	    strcat(zkc, "|");
	      return(yn);
	    }

            if(strncmp(wdw+ln-2, "", 2) == 0){ // ߋ
	      ptreplace(wdw+ln-2, 2, "");
	      strcat(zkc, "p");  yn = 1;
	    } else
            if(strncmp(wdw+ln-2, "", 2) == 0){
	      ptreplace(wdw+ln-2, 2, "");
	      strcat(zkc, "|");  yn = 1;
	    } else
            if(strncmp(wdw+ln-2, "", 2) == 0){
	      ptreplace(wdw+ln-2, 2, "");
	      strcat(zkc, "|");  yn = 1;
	    } else
            if(strncmp(wdw+ln-2, "", 2) == 0){
	      ptreplace(wdw+ln-2, 2, "");
	      strcat(zkc, "|");  yn = 1;
	    } else
	    // 16/4/28
          if(strncmp(wdw+ln, "", 2) < 0 || strncmp(wdw+ln, "", 2) > 0){
 if(Dbg == 'Y') fprintf(dbg, "--- simodanKatsu Nhira --- jpw = %s wdw = %s \n", jpw, wdw);
	    ptinsert(wdw+ln-2, "");
	    strcat(zkc, "");  yn = 1;
	    return(yn);
	  }

	    return(yn);
}

int godanKatsu(jpw)
char *jpw;
{
    int ln, i, j, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 4) return(yn);
    //    if(strncmp(wdw, "Ȃ", 8) == 0) return(yn);
    for(j = 0; j < 9; j++){ // _syuu
      if(strcmp(jpw+ln-2, godan_syuu[j]) == 0) break;
    }
    if(j == 9) return(yn);
    if(strcmp(jpw+ln-2, godan_syuu[j]) == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu --- jpw = %s gokan = %s \n", jpw, gokan);
	if(strncmp(wdw+ln-2, "", 6) == 0
        || strncmp(wdw+ln-2, "", 6) == 0 ){ //珟
	  for(i = 0; i< 9; i++){ // _syuu
	    if(strcmp(jpw+ln-2, godan_syuu[i]) == 0){
	      ptreplace(wdw+ln-2, 2, godan_syuu[i]);
	      yn = 1;
	      break;
	    }
	  }
	} else
	// 0: 1:h 2:񂾂 3:c񂾂 4:񂾂 5:󂵂
        for(i = 0; i < 6; i++){
          if(strncmp(wdw+ln-2, kako[i], 4) == 0 &&
          (strncmp(wdw+ln+2, "", 2) == 0 || strncmp(wdw+ln+2, "", 2) == 0)){
	    if(strcmp(jpw+ln-2, genz[i]) == 0){
	      ptreplace(wdw+ln-2, 2, genz[i]);
	      yn = 1;
	      break;
	    }
	  }
	}
        if(strncmp(wdw+ln-2, godan_mize[j], 2) == 0){ // 
          for(i = 0; i < 9; i++){ // go_a
            if(strncmp(wdw+ln, go_a[i], strlen(go_a[i])) == 0){
	      ptreplace(wdw+ln-2, 2, godan_syuu[j]);
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu go_a --- jpw = %s wdw = %s \n", jpw, wdw);
	      yn = 1;
	      break;
	    }
	  }
	} else
        if(strncmp(wdw+ln-2, godan_masu[j], 2) == 0){ // 
          if(strncmp(wdw, "Ȃ", 8) == 0) return(yn);
          if(strncmp(wdw, "K", 6) == 0) return(yn);
          for(i = 0; i < 26; i++){ // go_i
            if(strncmp(wdw+ln, go_i[i], strlen(go_i[i])) == 0){
	      ptreplace(wdw+ln-2, 2, godan_syuu[j]);
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu go_i --- jpw = %s wdw = %s \n", jpw, wdw);
              if(strncmp(wdw+ln, "A", 2) == 0) strcat(zkc, "|");
	      yn = 1;
	      return(yn); //	      break;
	    }
	  }
	  // ̕Ђ炪ȈȊO Rɓo J}ŋ؂Ȃꍇ
          if(strncmp(wdw+ln, "", 2) < 0 || strncmp(wdw+ln, "", 2) > 0){
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu go_i Nhira --- jpw = %s wdw = %s \n", jpw, wdw);
	    ptreplace(wdw+ln-2, 2, godan_syuu[j]);
	    strcat(zkc, "");  yn = 1;
	    return(yn);
	  }

        } else
        if(strncmp(wdw+ln-2, godan_kate[j], 2) == 0){ // 
          for(i = 0; i < 10; i++){ // go_e
            if(strncmp(wdw+ln, go_e[i], strlen(go_e[i])) == 0){
	      ptreplace(wdw+ln-2, 2, godan_syuu[j]);
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu go_e --- jpw = %s wdw = %s \n", jpw, wdw);
	      strcat(zkc, gz_e[i]);
	      yn = 1;
	      break;
	    }
	  }
        } else
        if(strncmp(wdw+ln-2, godan_miz2[j], 2) == 0){ // 
          if(strncmp(wdw, "͂", 6) == 0) return(yn);
          for(i = 0; i < 1; i++){ // go_o
            if(strncmp(wdw+ln, go_o[i], strlen(go_o[i])) == 0){
	      ptreplace(wdw+ln-2, 2, godan_syuu[j]);
 if(Dbg == 'Y') fprintf(dbg, "--- godanKatsu go_o --- jpw = %s wdw = %s \n", jpw, wdw);
	      yn = 1;
	      break;
	    }
	  }
          if(yn == 1 && strncmp(wdw+ln, "", 2) == 0){ // 
	    ptcut(wdw+ln, 2);	    strcat(zkc, "|");
	  }
        }


      }
    }
    return(yn);
}

int pastKatsu(jpw)
char *jpw;
{
    int ln, i, yn;
    char cmp[40];
    yn = 0;
    ln = strlen(jpw);
    if(ln < 4) return(yn);
      strncpy(cmp, jpw, ln-2);  cmp[ln-2] = '\0';
      if(strncmp(wdw, cmp, ln-2) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- pastKatsu --- jpw = %s cmp = %s \n", jpw, cmp);
        if(strncmp(wdw+ln-2, "", 6) == 0){ // Ȃ
	  	    if(strcmp(jpw+ln-2, "") == 0){
	  	      ptreplace(wdw+ln-2, 6, "");
	  	      strcat(zkc, "p");  yn = 1;
	  	    }
	} else
	if(strncmp(wdw+ln-2, "", 4) == 0){
	  for(i = 0; i< 9; i++){
	    if(strcmp(jpw+ln-2, godan_syuu[i]) == 0){
	      ptreplace(wdw+ln-2, 4, godan_syuu[i]);
	      strcat(zkc, "p");  yn = 1;
	      break;
	    }
	  }
	} else
	if(strncmp(wdw+ln-2, "", 4) == 0){
	  for(i = 0; i< 9; i++){
	    if(strcmp(jpw+ln-2, godan_syuu[i]) == 0){
	      ptreplace(wdw+ln-2, 4, godan_syuu[i]);
	      strcat(zkc, "|");  yn = 1;
	      break;
	    }
	  }
	} else
	// 0: 1:h 2: 3:c 4: 5:󂵂
        for(i = 0; i < 6; i++){
          if(strncmp(wdw+ln-2, kako[i], 4) == 0){
	    if(strcmp(jpw+ln-2, genz[i]) == 0){
	      ptreplace(wdw+ln-2, 4, genz[i]);
	      strcat(zkc, "p");  yn = 1;
	      break;
	    }
	  }
	}
	// 0: 1:h 2: 3:c 4: 5:󂵂
        for(i = 0; i < 6; i++){
          if(strncmp(wdw+ln-2, site[i], 4) == 0){
	    if(strcmp(jpw+ln-2, genz[i]) == 0){
	      ptreplace(wdw+ln-2, 4, genz[i]);
	      strcat(zkc, "|");  yn = 1;
 if(Dbg == 'Y') fprintf(dbg, "--- pastKatsu --- jpw = %s wdw = %s \n", jpw, wdw);
	      break;
	    }
	  }
	}
        if(strncmp(wdw+ln-2, "", 2) == 0){ // 
	    if(strcmp(jpw+ln-2, "") == 0){
	      ptreplace(wdw+ln-2, 2, "");
	      strcat(zkc, "p");  yn = 1;
	    }
	}
      }
      //    }
    return(yn);
}

int keiyoKatsu(jpw, hinsi)
char *jpw, *hinsi;
{
    int ln, i, j, yn = 0;
    char gokan[40];
    ln = strlen(jpw);
    if(ln < 4) return(yn);
    if(*hinsi != '5') return(yn);

    strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
    if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- keiyoKatsu 0 --- jpw = %s gokan = %s wdw = %s \n", jpw, gokan, wdw);
      // ꂽ
      if(strncmp(wdw+ln-2, "", 2) == 0 || strncmp(wdw+ln-2, "", 2) == 0){
	if(strncmp(wdw, "", 4) == 0) return(yn);
	if(strncmp(wdw, "", 4) == 0) return(yn);
	ptreplace(wdw+ln-2, 2, jpw+ln-2);
	    strcat(zkc, "|");  yn = 1;
	    return(yn);
      }
      if(strncmp(wdw+ln-2, "", 4) == 0){
	    ptinsert(wdw+ln-2, jpw+ln-2);
	    yn = 1;
	    return(yn);
      }
    }

    if(strcmp(jpw+ln-2, "") == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- keiyoKatsu --- jpw = %s gokan = %s wdw = %s \n", jpw, gokan, wdw);
       if(strncmp(wdw+ln-2, "", 6) == 0 // 
        || strncmp(wdw+ln-2, "肵", 6) == 0){ // 肵
	      ptreplace(wdw+ln-2, 6, "");
	      strcat(zkc, "p");  yn = 1;
	} else
        // 낤 ȂȂ 
        for(i = 0; i < 4; i++){ // ke_i
          if(strncmp(wdw+ln-2, ke_i[i], strlen(ke_i[i])) == 0){
	    ptreplace(wdw+ln-2, 2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- keiyoKatsu ke_i --- jpw = %s wdw = %s \n", jpw, wdw);
            if(strncmp(wdw+ln, "A", 2) == 0) strcat(zkc, "|");
	    yn = 1;
	    return(yn);
	  }
        }
      }
    } else
    if(strcmp(jpw+ln-2, "") == 0){ // 
      strncpy(gokan, jpw, ln-2);  gokan[ln-2] = '\0';
      if(strncmp(wdw, gokan, ln-2) == 0){ // Ov
 if(Dbg == 'Y') fprintf(dbg, "--- keiyoKatsu --- jpw = %s gokan = %s wdw = %s \n", jpw, gokan, wdw);
	// Â Â낤 ͎

	if(strncmp(wdw+ln-2, "", 6) == 0 ){
	    ptinsert(wdw+ln-2, "");
	    strcat(zkc, "p");  yn = 1;
	    return(yn);
	}
	if(strncmp(wdw+ln-2, "ł", 4) == 0 ){
	    ptinsert(wdw+ln-2, "");
	    yn = 1;
	    return(yn);
	}
	if(strncmp(wdw+ln-2, "ɁB", 4) == 0 ){
	  ptreplace(wdw+ln-2, 2, "");
	  strcat(zkc, "|");  yn = 1;
	    return(yn);
	}
        // ÂłȂ ÂɂȂ ÂȂ
        for(i = 0; i < 6; i++){ // keDa
          if(strncmp(wdw+ln-2, keDa[i], strlen(keDa[i])) == 0){
	    ptreplace(wdw+ln-2, 2, "");
 if(Dbg == 'Y') fprintf(dbg, "--- keiyoKatsu keDa --- jpw = %s wdw = %s \n", jpw, wdw);
	    yn = 1;
	    return(yn);
	  }
        }
      }
    }
    return(yn);
}


int desumasu_ck()
{
    int  i;
    char wdw2[JSentLen];

  if(Dbg == 'Y') fprintf(dbg, "--- desumasu_ck --- 1 wdw = %s \n", wdw);
  /*
    if( strncmp(wdw, "ł", 4) == 0 || strncmp(wdw, "", 4) == 0){
      if(strncmp(wdw, "ł", 6) == 0) return 0;
      ptreplace(wdw, 4, "ł");
      if(strncmp(wdw, "ł", 4) == 0) strcpy(zkc, "|h");
      strcpy(jwd, "ł");
      return 1;
      }*/
    if( strncmp(wdw, "̂ł", 6) == 0 || strncmp(wdw, "ł", 6) == 0){
      ptreplace(wdw, 6, "ł");
      strcpy(jwd, "ł");
      return 1;
    }

    if( strncmp(wdw, "ł", 6) == 0){
      ptreplace(wdw, 6, "ł");
      strcpy(zkc, "p|h");
      strcpy(jwd, "ł");
      return 1;
    }
//    if(strncmp(wdw, "낤", 6) == 0){
//      ptcut(wdw, 6);
//      strcpy(wdh, wdw);
//      for(i = 0; ;i++){
//        if(aj[wnb-1][i][0] == '\0') break;
//        strcat(az[wnb-1][i], "V");
//      }
//      strcpy(wdx, "(exit)");
//      return 1;
//    }
    if( (strncmp(wdw, "", 2) == 0
    || strncmp(wdw, "Ȃ̂", 6) == 0
    || strncmp(wdw, "", 2) == 0)
    && strncmp(wdw+2, "", 2) ){
      if(strncmp(wdw, "炵Ȃ", 10) == 0) return 0;
      if(strncmp(wdw, "낤", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "炯", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0 || strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      ptreplace(wdw, 2, "ł");
      strcpy(jwd, "ł");
      if(strstr(aj[wnb-1][0], "wh") && strchr(az[wnb-1][0], 'q')) ptinsert(wdw+6, "");
      return 1;
    }
    if(strncmp(wdw, "܂", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "ph");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "B", 4) == 0){
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "p");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "܂", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        if(strchr(az[wnb-1][i], 'v') && strstr(az[wnb-2][i], ""))
          strcat(az[wnb-2][i], "");
        strcat(az[wnb-1][i], "Nh");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "Ȃ", 4) == 0 || strncmp(wdw, "", 4) == 0){
      if(strncmp(wdw, "ȂŁA", 12) == 0) return 0;
      if(strncmp(wdw, "Ȃ͂", 10) == 0) return 0;
      if(strncmp(wdw, "Ȃ킯", 8) == 0) return 0;
      if(strncmp(wdw, "Ȃ", 8) == 0) return 0;
      if(strncmp(wdw, "Ȃ", 8) == 0) return 0;
      if(strncmp(wdw, "Ȃ", 6) == 0) return 0;
      if(strncmp(wdw, "Ȃ", 6) == 0) return 0;
      if(strncmp(wdw, "Ȃ", 6) == 0) return 0;
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "N");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 2) == 0 || strncmp(wdw, "", 2) == 0){
      if(strncmp(wdw, "Ԃ", 8) == 0) return 0;
      if(strncmp(wdw, "܂", 8) == 0) return 0;
      if(strncmp(wdw, "", 8) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "", 6) == 0) return 0;
      if(strncmp(wdw, "ʂ", 6) == 0) return 0;
      if(strncmp(wdw, "ʂ", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      if(strncmp(wdw, "", 4) == 0) return 0;
      strcpy(wdw2, wdw);
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "N");
	if(strncmp(wdw2, "", 2) == 0) strcat(az[wnb-1][i], "|");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "܂傤", 8) == 0){
      ptcut(wdw, 8);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "V");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "܂", 4) == 0){
      if(strncmp(wdw, "܂", 8) == 0) return 0;
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "VN");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    /*    if(
//       strncmp(wdw, "낤", 4) == 0 ||
       strncmp(wdw, "", 4) == 0){
      if(strncmp(wdw+4, "", 4) == 0) return 0;
      if(strncmp(wdw+4, "", 4) == 0) return 0;
      if(strncmp(wdw+4, "", 4) == 0) return 0;
      if(strncmp(wdw+4, "x", 4) == 0) return 0;
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "V");
      }
      strcpy(wdx, "(exit)");
      return 1;
      }*/
    if(strncmp(wdw, "܂", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|hJ");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "܂", 4) == 0){
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|hJ");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 4) == 0 || strncmp(wdw, "", 4) == 0){
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 8) == 0
    || strncmp(wdw, "", 8) == 0){
      ptcut(wdw, 8);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 2) == 0
    && strncmp(wdw, "΂", 4) && strncmp(wdw, "΂", 4) ){
      if(strncmp(wdw, "΂", 6) == 0) return 0;
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|IF");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "Ă", 10) == 0){
      ptcut(wdw, 10);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "ĉ", 8) == 0){
      ptcut(wdw, 8);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 8) == 0){
      ptcut(wdw, 8);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "", 4) == 0 || strncmp(wdw, "݂", 4) == 0){
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "Ȃ", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "ȁB", 4) == 0 || strncmp(wdw, "ȁA", 4) == 0
    || strncmp(wdw, "Ȃ", 4) == 0 || strncmp(wdw, "Ȃ", 4) == 0){
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "N|");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "ˁB", 4) == 0 || strncmp(wdw, "ˁv", 4) == 0){
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|t");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "܂", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    /*    if(strncmp(wdw, "܂", 8) == 0){
      ptcut(wdw, 8);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "P");
      }
      strcpy(wdx, "(exit)");
      return 1;
      }  */
    if(strncmp(wdw, "Ă܂܂", 14) == 0){
      ptcut(wdw, 14);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "P|h");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "Ƃ", 10) == 0){
      if(strncmp(wdw, "Ƃ", 12) == 0) return 0;
      ptcut(wdw, 10);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "P|o");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    if(strncmp(wdw, "Ƃ܂", 14) == 0){
      ptcut(wdw, 14);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "P|oh");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    /*    if(strncmp(wdw, "炢", 6) == 0){
      ptcut(wdw, 6);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "");
      }
      strcpy(wdx, "(exit)");
      return 1;
      }*/
/*    if(strncmp(wdw, "", 4) == 0 || strncmp(wdw, "ꂽ", 4) == 0){
      ptcut(wdw, 4);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "");
      }
      strcpy(wdx, "(exit)");
      return 1;
    } */
    if(strncmp(wdw, "B", 4) == 0
    && strncmp(wdw+2, "", 2) && strncmp(wdw+2, "", 2)
    && strncmp(wdw+2, "", 2) && strncmp(wdw+2, "", 2)
    && strncmp(wdw+2, "", 2) && strncmp(wdw+2, "", 2) ){
      if(strncmp(wdw, "", 6) == 0) return 0;
      ptcut(wdw, 2);
      strcpy(wdh, wdw);
      for(i = 0; ;i++){
        if(aj[wnb-1][i][0] == '\0') break;
        strcat(az[wnb-1][i], "|g");
      }
      strcpy(wdx, "(exit)");
      return 1;
    }
    return 0;
}

diclook(ppt)
int  ppt;
{
  int  ln, nx, j, yn;
    char jew[170], jpw[JWordLen], egw[WordLen], zkw[ZokuLen];
    char *jw, *jt, *pts;
    char eg2[300], hinsi[2], chr[3];
    nx = 1;
    if(ppt < 0) ppt = 1;
    while(nx > 0){
      dictread(ppt, &nx);

      while(j = instr(jer, jwd)){
        decompose(jer, j, jew);
        jw = (char*)strchr(jew, ';');
        ptcpy(jpw, jew, jw);

	if(pts = strchr(jpw, '#')){
 if(Dbg == 'Y') fprintf(dbg, "--- diclook --- jpw = %s \n", jpw);
          umeJukugo(pts, jpw, wdw, &yn);
	  if(yn){
            if(isdigit(*(pts+1))) strcpy(eg2, pts+2);
            else strcpy(eg2, pts+1);
	    strcpy(wdx, jpw); // 16/3/16
	    left(jpw, wdw, strstr(wdw, eg2)+strlen(eg2)-wdw);
 if(Dbg == 'Y') fprintf(dbg, "--- diclook --- wdx = %s jpw = %s wdw = %s \n", wdx, jpw, wdw);
	  }
        }
 
	// if(Dbg == 'Y') fprintf(dbg, "--- diclook --- nx = %d jpw = %s wdw = %s \n", nx, jpw, wdw);
	// `Ap̎AP͍̗pȂ
	if(ct < 2 && strlen(jpw) == 2) goto NTT;
        if(ct == 1 && strncmp(wdw, jpw, strlen((char*)jpw)) != 0){ // 16/4/20
	  hinsi[0] = *(jw+1);  hinsi[1] = '\0';
	  if(strchr("345", *hinsi)){ //  `
	  // if(Dbg == 'Y') fprintf(dbg, "--- diclook % --- jpw = %s jew = %s \n", jpw, jew);
            if(postConj(jpw, hinsi) == 0) goto NTT;
	    goto DICL; // 16/4/20
	  }
          else goto NTT;
	}

	// `̎̕uv`uv̎ATϊp uv`uAv̎Aiܒi
	if(strncmp(wdw+strlen(jpw), "Ȃ", 4) == 0);  // 16/10/15
	else                                           // 16/10/15
	strncpy(chr, wdw+strlen(jpw), 2); chr[2] = '\0';
	if(ct == 0 && (*(jw+1) == '1' || *(jw+1) == '5') &&
	strstr("ĂȂ܂A", chr)) goto NTT;
	// Ŏn܂镡̏ꍇ
	if(ct == 0 && ptncmp(genSkip8, wdw, 8)) goto NTT;
	if(ct == 0 && ptncmp(genSkip6, wdw, 6)) goto NTT;
	if( strncmp(wdw, jpw, strlen(jpw)) != 0) goto NTT; // 16/4/20

      DICL:
	ct = 3; // 16/4/20
        strcpy(jwd, jpw);
        jt = (char*)strchr(jw+1, ';');
        ptcpy(egw, jw+1, jt);
        ln = strlen(jt+1);
        ptcpy(zkw, jt+1, jt+ln);
        yakSaiyo(jpw, egw, zkw);
 if(Dbg == 'Y') fprintf(dbg, "--- diclook --- jpw = %s egw = %s zkw = %s zkc = %s caz[%d] = %s \n", jpw, egw, zkw, zkc, cnb, caz[cnb]);
        NTT:
        cutoff(jer, 0, strlen(jew));
      }
      if(ppt < nx) ppt = nx;
    }
}

int ptcal(unsigned char *buf)
{
    int  ppt;
    unsigned int ptr;
    int code, code2;
    ppt = 5;
    ptr = (unsigned int)*buf * 256 + (unsigned int)*(buf+1);
    ptr = ptr & 0xffff;
    code = sj2j(ptr);    // sjis -> jis
    code -= 0x2121;
    code2 = code & 0x0f;
    code = code / 16;    // zyoui 3 keta
    ppt = code;
    if(code < 0) ppt = -5;
    else if(code < 48) ppt = code; 
    // `  48->130
    else if(code <= 53){
      ppt = 48 + (code - 48) * 16 + code2;
    } 
    ppt++;
      if(Dbg == 'Y') fprintf(dbg, "\n ptr = %4x code = %d ppt = %d \n", ptr, code, ppt);
    return(ppt);
}

int sj2j(unsigned int k)
{
    int k1;
    k1 = (k >> 8) & 0xff;
    k &= 0xff;
    if(k1 >= 0xe0) k1 -= 0x40;
    k1 = (k1-0x81) * 2 + 0x21;
    if(k > 0x7f) k--;
    if(k > 0x9d){
      k1++;
      k -= 0x9e - 0x21;
    }
    else{
      k -= 0x40 - 0x21;
    }
    return((k1 << 8) + k);
  }

dictread(ppt, nx)
int  ppt, *nx;
{
    int  len;
    long ptl;
    char *ptc, yn;
    ptl = (long)(ppt-1)*Size;
    rewind(jed);
    if(fseek(jed, ptl, 0)){
      printf("Seek error\n");
      exit(1);
    }
    fread(jer, Size-4, 1, jed);
    fscanf(jed, "%4d", nx);
    *(jer+Size-4) = '\0';
    if(ptc = (char*)strrchr(jer, ':')) *(ptc+1) = '\0';
    else{
      if(*jer && jer != (char*)strstr(jer, "              ")){
        printf("Abnormal  jer = %s \n", jer);
        exit(1);
      }
    }
}

decompose(jer, j, jew)
int  j;
char jer[], jew[];
{
    int  i;
    i = 0;
    while(jer[i] != ':'){
      jew[i] = jer[i];
      i++;
      if(i >= JWordLen*2+WordLen+ZokuLen){
        jew[i] = '\0';
        printf("Too long!! \n");
        exit(1);
      }
    }
    jew[i++] = ':';
    jew[i] = '\0';
}

yakSaiyo(jpw, egw, zkw)
char jpw[], egw[], zkw[];
{
  char *pt1, *pt2, zoku[200], zkc2[100], zkw2[100];

  // if(Dbg == 'Y') fprintf(dbg, "--- yakSaiyo --- wdx = %s jpw = %s egw = %s zkw = %s zkc = %s \n", wdx, jpw, egw, zkw, zkc);
    if(strcmp(jpw, wdx) == 0){
      cnb++;
      if(cnb > YakMaxCt){
        printf("Too many yakugo! \n");
        return;
      }
      strcpy(caj[cnb], egw);
 if(Dbg == 'Y') fprintf(dbg, "--- yakSaiyo --- caj[%d] = %s \n", cnb, caj[cnb] );
      if(zkc[0] != '\0'){
	//        strcpy(caz[cnb], zkc);
	//        strcat(caz[cnb], zkw);
        strcpy(zkc2, zkc);
        strcpy(zkw2, zkw);
        if(pt1 = strchr(zkc2, '|')) *pt1 = '\0';  // zkc
        if(pt2 = strchr(zkw2, '|')) *pt2 = '\0';  // zkw
        *zoku = '\0';
        if(pt1 && *(pt1+1) != '\0') strcpy(zoku, pt1+1); // zkc_add
        if(pt2 && *(pt2+1) != '\0') strcat(zoku, pt2+1); // zkw_add
        strcpy(caz[cnb], zkc2);
        strcat(caz[cnb], zkw2);
        if(pt1 || pt2) strcat(caz[cnb], "|");
        strcat(caz[cnb], zoku);
      }
      else strcpy(caz[cnb], zkw);
      return;
    }
    if(!strchr(egw, '#')) 
      strcpy(wdx, jpw);
    strcpy(caj[0], egw);
    if(zkc[0] != '\0'){
      //      strcpy(caz[0], zkc);
      //      strcat(caz[0], zkw);
      
      strcpy(zkc2, zkc);
      strcpy(zkw2, zkw);
      if(pt1 = strchr(zkc2, '|')) *pt1 = '\0';  // zkc
      if(pt2 = strchr(zkw2, '|')) *pt2 = '\0';  // zkw
      *zoku = '\0';
      if(pt1 && *(pt1+1) != '\0') strcpy(zoku, pt1+1); // zkc2
      if(pt2 && *(pt2+1) != '\0') strcat(zoku, pt2+1); // zkw2
      strcpy(caz[0], zkc2);
      strcat(caz[0], zkw2);
      if(pt1 || pt2) strcat(caz[0], "|");
      strcat(caz[0], zoku);
      
    }
    else strcpy(caz[0], zkw);
}

int Conj()
{
    char hcode;
    char hcode2;
    int  i;
    if(strcmp(jwd, "") == 0 || strcmp(jwd, "") == 0){
      hcode = aj[wnb-1][0][0];
      hcode2 = aj[wnb-1][1][0];
  if(Dbg == 'Y') fprintf(dbg, "-- Conj -- hcode = %c hcode2 = %c \n", hcode, hcode2);
      if(strchr("12678%", hcode) || strchr("12678%", hcode2) || strcmp(aj[wnb-1][0], "0)") == 0){
        for(i = 0; ; i++){
          if(aj[wnb-1][i][0] == '\0') break;
          strcat(az[wnb-1][i], jwd);
        }
        strcpy(caj[0], "S");
        strcpy(wdx, jwd);
        return 1;
      }
    }
    if(strcmp(jwd, "") == 0){
      hcode = aj[wnb-1][0][0];
      if(strchr("35", hcode)){
        if(strchr(az[wnb-1][0], 'j')) return 0;
        for(i = 0; ; i++){
          if(aj[wnb-1][i][0] == '\0') break;
          strcat(az[wnb-1][i], jwd);
        }
        strcpy(caj[0], "O");
        strcpy(wdx, jwd);
        return 1;
      }
    }
    if(strcmp(jwd, "") == 0){
      hcode = aj[wnb-1][0][0];
      if(strchr("37", hcode)){
        for(i = 0; ; i++){
          if(aj[wnb-1][i][0] == '\0') break;
          strcat(az[wnb-1][i], "p");
        }
        strcpy(caj[0], "O");
        strcpy(wdx, jwd);
        return 1;
      }
    }
    return 0;
}

japcopy()
{
    int  i;
    strcpy(caj[0], "1");    strcpy(caz[0], "|");
    strcpy(caj[1], "5");    strcpy(caz[1], "|");
    strcpy(caj[2], "6");    strcpy(caz[2], "|");
    for(i = 0; i <= 2; i++) strcat(caj[i], jwd);
    caj[3][0] = '\0';    cnb = 2;
}

int irregVerb(sw)
int sw;
{
    FILE *irreg;
    char irdata[55], wdd[WordLen], juk;
    char og[32], p1[32], *p2, *ogpt, *ptn;
    int  find = 0;

    strcpy(wdd, wd);
    if(ptn = (char*)strchr(wdd, ' ')){ *ptn = '\0'; juk = 'y'; }
    else juk = 'n';
    irreg = fopen("IRREGUL", "r");
    if(irreg == 0){ printf("IRREGUL not exist! \n"); return; }
    while(fgets(irdata, 53, irreg)){
      if(ptn = (char*)strchr(irdata, ';')) *ptn = '\0';
      if(ogpt = (char*)strchr(irdata, ',')) ptcpy(og, irdata, ogpt);
      if(p2 = (char*)strchr(ogpt+1, ',')) ptcpy(p1, ogpt+1, p2++);
      if(strcmp(wdd, og) == 0){
        find = 1;
	if(sw == 1){ strcpy(wdd, p1); break; }
	if(sw == 2){ strcpy(wdd, p2); break; }
      }
    }
    fclose(irreg);
    if(juk == 'y'){
      ptn = (char*)strchr(wd, ' ');
      strcat(wdd, ptn);
    }
    strcpy(wd, wdd);    return(find);
}

// ߍ݌^n̏
//
umeJukugo(pts, egw, wdw, yn)
int  *yn;
char *pts, egw[45], *wdw;
{
    int  j2;
    char eg1[40], eg2[40], eg3[40], *ps2;

    ptcpy(eg1, egw, pts);        // eg1 = egw̒#O̕
    if(isalnum(*(pts+1))) strcpy(eg2, pts+2);
    else strcpy(eg2, pts+1);
    if(wdw != (char*)strstr(wdw, eg1) ){
      *yn = 0;	return;
    }
    //    if(ps2 = (char*)index(eg2, '#')){
    //      if(*(ps2+1) == '5') strcpy(eg3, ps2+2);
    //      else strcpy(eg3, ps2+1);
    //      *(ps2-1) = '\0';
    //    }
 if(Dbg == 'Y') fprintf(dbg, "--- umeJukugo --- %s %s %s \n", wdw, eg1, eg2);
    umeck(wdw, eg1, eg2, pts, 0, &j2, yn);
    if(*yn == 0) return;
    //    if(ps2) umeck(wdw, eg2, eg3, ps2, j2, &j2, yn);
}

umeck(wdw, ed, e2, ps, l0, j2, yn)
int  l0, *j2, *yn;
char wdw[100], ed[40], e2[40], *ps;
{
    int  l1, jc, jp, jq, ll, sp;
    char z, rm[50];
    l1 = strlen(ed)+l0;
    while((char) 1){
      *j2 = ninstr(l1, wdw, e2);  // 7
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- wdw+l1 = %s e2 = %s *j2 = %d \n", wdw+l1, e2, *j2);
//  fprintf(prcs, "--- umeck --- wdw = %s ed = %s e2 = %s ps = %s j2 = %d \n", wdw, ed, e2, ps, *j2);
//  fprintf(prcs, "--- umeck --- l0 = %d l1 = %d ll = %d ps = %s \n", l0, l1, ll, ps);
      if(*j2 == 0 ){
	//      if(*j2 == 0 || (*(ps+1) != ' ' && *j2 == l1+1)){
	*yn = 0;	return;
      }
 //      if(wdw[*j2-2] == ' '){
	jc = ninstr(l1, wdw, "A");
	jp = ninstr(l1, wdw, "B");
	jq = ninstr(l1, wdw, "H");
	if((jc > 0 && jc < *j2) || (jp > 0 && jp < *j2) || (jq > 0 && jq < *j2)){
	  *yn = 0;	return;
	}
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- l1 = %d \n", l1);
	jp = l1;
	sp = 0;
	//--> 
	while(jp = ninstrc(++jp, wdw, ' ')){
	  if(jp < *j2){
	    sp++;
	    if(sp > 5){ *yn = 0;	return; }
	  }
	  else break;
	}
	//--
        ll = *j2 + strlen(e2);
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- ll = %d \n", ll);
 break; // 16/3/17

//  fprintf(prcs, "--- umeck --- strlen(wdw) = %d \n", strlen(wdw));
        if((strlen(wdw) != ll-1 || (ps > 0 && strcmp(ed, e2) != 0)) &&
        wdw[ll-1] != ' '){
//  fprintf(prcs, "--- umeck --- strlen(wdw) = %d \n", strlen(wdw));
          if(wdw[ll-1] == '\0') break;    // ̂Psǉ
	  if(wdw[ll-1] == '.' && strstr(wdw, "h") && isupper(wdw[ll+1])){
	    insertc(wdw, ll-1, ' ', strlen(wdw));	break;
	  }
	  else l1 = ll;
        }
	else break;
	//      }
	//      else l1 = *j2+1;
    }
    if(*(ps+1) == ' '){
      *yn = 1;	return;
    }
    z = *(ps+1);
    mid(wdw, l1+1, *j2-l1-2, rm);
    mid(wdw, l1+1, *j2-l1-1, rm);
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- rm = %s \n", rm);
    if((z == '2' || z == '8') && *rm && index(rm, ' ')){
      *yn = 0;	return;
    }
    if(isdigit(z) && *rm && z != '8'){ *yn = 1;	return; }
    pronounCk(z, rm, yn);
 if(Dbg == 'Y') fprintf(dbg, "--- umeck --- >%s %s \n", ps, rm);
}

/*
codeCk(jpw, egw)
char jpw[], egw[];
{
    char code, dums[5];
    ct = -1;	code = jpw[0];
    if((zkc == 'p' || zkc == 'G') && code != '3' && code != '4') return;
    if((zkc == 'C' || zkc == 'U') && code != '5' && code != '6') return;
    if(zkc == 'x' && code != '1' && code != '3' && code != '4'){
      if(code != '5' || strlen(egw) < 6) return;
      else{
	right(egw, dums, 3);
	if(strcmp(dums, " of") != 0) return;
      }
    }
    if(zkc == 'l' && code != '5') return;
    ct = 0;
}
*/

pronounCk(z, rm, yn)
int *yn;
char z, rm[50];
{
    int  ln;
    if(z == '8' && (strcmp(rm, "a")==0 || strcmp(rm, "an")==0 ||
    strcmp(rm, "the")==0 || strcmp(rm, "this")==0 || strcmp(rm, "that")==0)){
      *yn = 1;	return;
    }
    if(z == 'h' && (strcmp(rm, "me")==0 || strcmp(rm, "you")==0 ||
    strcmp(rm, "him")==0 || strcmp(rm, "her")==0 || strcmp(rm, "us")==0 ||
    strcmp(rm, "them")==0)){
      *yn = 1;	return;
    }
    ln = strlen(rm);
    if(z == 's' && (instr(rm, "self")==ln-3 || instr(rm, "selves")==ln-5)){
      *yn = 1;	return;
    }
    if(z == 'D' && (strcmp(rm, "it")==0 || strcmp(rm, "this")==0 ||
    strcmp(rm, "that")==0 || strcmp(rm, "them")==0 || strcmp(rm, "these")==0 ||
    strcmp(rm, "those")==0)){
      *yn = 1;	return;
    }
    if(z == 'p' || z == '8'){
      if(strcmp(rm, "my")==0 || strcmp(rm, "your")==0 || strcmp(rm, "his")==0 ||
      strcmp(rm, "her")==0 || strcmp(rm, "its")==0 || strcmp(rm, "our")==0 ||
      strcmp(rm, "their")==0){
        *yn = 1;	return;
      }
      if(ln >= 3 &&
      (ninstr(ln, rm, "'s")==ln-1 || ninstr(ln-1,rm, "s'")==ln-1)){
	*yn = 1;	return;
      }
    }
    *yn = 0;
}

/*
int getline(char *ptr, int maxlen)
{
    int  n;
    char c;
    for(n = 1; n < maxlen; n++){
      c = getchar();
      if(c == '\n') break;
      *ptr++ = c;
    }
    *ptr = '\0';
    return(n);
  }


getno(){
    int  i, ia, iz, in, eq, ln;
    char wd1[WordLen];
    ia = wnb;
    for(i = 0; i <= wnb; i++)
      if(strcmp(egw[i], wd) == 0){
        ia = i;    break;
      }
    iz = ia+WdDisp;
    if(iz > wnb) iz = wnb;
    if(ia == -1) in = 0;
    else{
      strcpy(wd1, wd);
      for(i = 0; i <= iz; i++){
        if(strlen(egw[i]) < strlen(wd1)){
          ln = strlen(egw[i]);
          if(strncmp(wd1, egw[i], ln) == 0){ eq = 2;  break; }
        }
      }
      if(eq != 0){
        if(i < 0) i = 0;
        in = i;
      }
    }
  printf("--- getno egw[%d] = %s  in = %d  wnb = %d \n", in, egw[in], in, wnb);
    return(in);
  }
*/
